package com.huiju.estateagents.job;//package com.huiju.estateagents.job;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.huiju.estateagents.base.BaseController;
import com.huiju.estateagents.common.CommConstant;
import com.huiju.estateagents.common.DateUtils;
import com.huiju.estateagents.redpack.entity.TaOrgOrder;
import com.huiju.estateagents.redpack.mapper.TaOrgOrderMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;

/**
 * 订单相关操作
 */
@Component
@Configuration
@EnableScheduling
public class OrgOrderTimeJob extends BaseController {

    private Logger logger = LoggerFactory.getLogger(OrgOrderTimeJob.class);

    @Autowired
    private TaOrgOrderMapper taOrgOrderMapper;

    /**
     * 处理过期订单
     */
    @Scheduled(cron = "0 0/5 * * * ?")
    private void dealExpireOrder() {
        logger.info("处理过期订单 定时任务开始");

        try {
            TaOrgOrder taOrgOrder = new TaOrgOrder();
            taOrgOrder.setTradingStatus(CommConstant.TRADING_STATUS_FAIL);

            UpdateWrapper<TaOrgOrder> taOrgOrderUpdateWrapper = new UpdateWrapper<>();
            taOrgOrderUpdateWrapper.eq("status", CommConstant.STATUS_NORMAL);
            taOrgOrderUpdateWrapper.lt("expiry_date", DateUtils.localDateTimeToString(LocalDateTime.now()));
            taOrgOrderUpdateWrapper.eq("trading_status",CommConstant.TRADING_STATUS_PROCESSING);
            Integer result = taOrgOrderMapper.update(taOrgOrder, taOrgOrderUpdateWrapper);
            logger.info("处理过期订单 共更新" + result + "条数据");
        } catch (Exception e) {
            logger.error("处理过期订单 异常", e);
        }
        logger.info("处理过期订单 定时任务结束");
    }

}