package com.huiju.estateagents.miniPackage.service.impl;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.bo.request.SaveCzOrderRequestBO;
import com.huiju.estateagents.center.taUser.entity.TaUser;
import com.huiju.estateagents.center.taUser.mapper.TaUserMapper;
import com.huiju.estateagents.common.CommConstant;
import com.huiju.estateagents.common.CommonUtils;
import com.huiju.estateagents.common.wxpay.WXPay;
import com.huiju.estateagents.common.wxpay.WXPayConstants;
import com.huiju.estateagents.common.wxpay.WxConfig;
import com.huiju.estateagents.entity.TaOrg;
import com.huiju.estateagents.mapper.TaOrderMapper;
import com.huiju.estateagents.miniPackage.entity.SysVipPackage;
import com.huiju.estateagents.miniPackage.entity.TaUserPackages;
import com.huiju.estateagents.miniPackage.mapper.SysVipPackageMapper;
import com.huiju.estateagents.miniPackage.mapper.TaUserPackagesMapper;
import com.huiju.estateagents.miniPackage.service.ITaUserPackagesService;
import com.huiju.estateagents.redpack.entity.TaOrgAccount;
import com.huiju.estateagents.redpack.entity.TaOrgOrder;
import com.huiju.estateagents.redpack.entity.TaOrgOrderDetail;
import com.huiju.estateagents.redpack.mapper.TaOrgAccountMapper;
import com.huiju.estateagents.redpack.mapper.TaOrgOrderDetailMapper;
import com.huiju.estateagents.redpack.mapper.TaOrgOrderMapper;
import com.huiju.estateagents.redpack.service.ITaOrgOrderService;

/**
 * <p>
 * 会员套餐记录  服务实现类
 * </p>
 *
 * @author wxx
 * @since 2020-06-16
 */
@Service
public class TaUserPackagesServiceImpl extends ServiceImpl<TaUserPackagesMapper, TaUserPackages> implements ITaUserPackagesService {

    private final Logger logger = LoggerFactory.getLogger(TaUserPackagesServiceImpl.class);

    @Value("${pay-notify}")
    private String payNotify;

    @Autowired
    private ITaOrgOrderService iTaOrgOrderService;

    @Autowired
    private TaUserMapper taUserMapper;

    @Autowired
    private TaOrgAccountMapper taOrgAccountMapper;

    @Autowired
    private TaUserPackagesMapper taUserPackagesMapper;

    @Autowired
    private SysVipPackageMapper sysVipPackageMapper;

    @Autowired
    private TaOrgOrderMapper taOrgOrderMapper;

    @Autowired
    private TaOrgOrderDetailMapper taOrgOrderDetailMapper;

    /**
     * 购买套餐生成订单
     * @param saveCzOrderRequestBO
     * @param orgId
     * @param userId
     * @param ip
     * @return
     * @throws Exception
     */
    @Override
    @Transactional
    public ResponseBean saveBuyPackageOrder(SaveCzOrderRequestBO saveCzOrderRequestBO, Integer orgId, Integer userId, String ip) throws Exception {
        ResponseBean responseBean = new ResponseBean();

        SysVipPackage sysVipPackage = sysVipPackageMapper.selectById(saveCzOrderRequestBO.getPackId());
        if (sysVipPackage == null){
            responseBean.addError("配置不存在");
            return responseBean;
        }

        // 获取下单人名称
        TaUser taUser = taUserMapper.selectById(userId);
        if (taUser == null) {
            responseBean.addError("fail");
            return responseBean;
        }

        //校验账户余额
        if (CommConstant.ACCOUNT_BALANCE_PAY.equals(saveCzOrderRequestBO.getPayType())){
            if (calculateAmountFlag(orgId, saveCzOrderRequestBO.getPackId())){
                responseBean.addError("账户余额不足");
                return responseBean;
            }else {
                //更新账户余额
                TaOrgAccount taOrgAccount = assembleOrgAccount(orgId, saveCzOrderRequestBO.getAmount());
                QueryWrapper<TaOrgAccount> taOrgAccountQueryWrapper = new QueryWrapper<>();
                taOrgAccountQueryWrapper.eq("org_id", orgId);
                taOrgAccountMapper.update(taOrgAccount, taOrgAccountQueryWrapper);
            }
        }

        //构造订单记录
        TaOrgOrder taOrgOrder = assembleTaOrgOrder(orgId, saveCzOrderRequestBO, taUser);
        if (taOrgOrderMapper.insert(taOrgOrder) < 1){
            responseBean.addError("插入订单记录失败");
            return responseBean;
        }

        //构造订单详情记录
        TaOrgOrderDetail taOrgOrderDetail = assembleTaOrgOrderDetail(taOrgOrder);
        if (taOrgOrderDetailMapper.insert(taOrgOrderDetail) < 1){
            responseBean.addError("插入订单详情记录失败");
            return responseBean;
        }

        //构造会员套餐记录订单
        TaUserPackages taUserPackages = assembleTaUserPackages(orgId, userId, saveCzOrderRequestBO, taOrgOrder.getOrderId());
        if (taUserPackagesMapper.insert(taUserPackages) < 1){
            responseBean.addError("插入会员套餐记录订单失败");
            return responseBean;
        }

        //请求微信接口获取支付二维码
        if (CommConstant.WX_PAY.equals(saveCzOrderRequestBO.getPayType())){
            ResponseBean payResult = iTaOrgOrderService.payUnifiedOrder(taOrgOrder, ip, CommConstant.ITEM_TYPE_PACKAGE);
            if (ResponseBean.CODE_SUCCESS != payResult.getCode()) {
                return payResult;
            }

            JSONObject obj = new JSONObject();
            obj.put("amount", sysVipPackage.getPrice());
            obj.put("realAmount", sysVipPackage.getPrice());
            obj.put("serviceFee", sysVipPackage.getPrice());
            obj.put("mchPayRate", CommConstant.WXMCH_PAY_RATE_STRING);
            obj.put("payQrCode", payResult.getData());
            obj.put("orderInfo", taOrgOrder);
            responseBean.addSuccess(obj);
        }

        return responseBean;

    }

    private TaOrgAccount assembleOrgAccount(Integer orgId, Integer amount){
        QueryWrapper<TaOrgAccount> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("org_id", orgId);
        TaOrgAccount selectTaOrgAccount = taOrgAccountMapper.selectOne(queryWrapper);

        TaOrgAccount taOrgAccount = new TaOrgAccount();
        taOrgAccount.setOrgId(orgId);
        taOrgAccount.setAvailableBalance(selectTaOrgAccount.getAvailableBalance() - amount);
        taOrgAccount.setRealBalance(selectTaOrgAccount.getRealBalance() - amount);
        taOrgAccount.setPurchaseAmount(selectTaOrgAccount.getPurchaseAmount() + amount);

        return taOrgAccount;
    }

    private boolean calculateAmountFlag(Integer orgId, Integer packId){
        //查询账户余额
        QueryWrapper<TaOrgAccount> taOrgAccountQueryWrapper = new QueryWrapper<>();
        taOrgAccountQueryWrapper.eq("org_id", orgId);
        TaOrgAccount taOrgAccount = taOrgAccountMapper.selectOne(taOrgAccountQueryWrapper);

        //查询套餐价格
        SysVipPackage sysVipPackage = sysVipPackageMapper.selectById(packId);
        if (taOrgAccount == null || taOrgAccount.getAvailableBalance() < sysVipPackage.getPrice()){
            return true;
        }

        return false;
    }

    public TaOrgOrder assembleTaOrgOrder(Integer orgId, SaveCzOrderRequestBO saveCzOrderRequestBO, TaUser taUser){
        SysVipPackage sysVipPackage = sysVipPackageMapper.selectById(saveCzOrderRequestBO.getPackId());

        // 1.创建订单主表信息
        TaOrgOrder taOrgOrder = new TaOrgOrder();
        taOrgOrder.setOrgId(orgId);
        //类型为套餐缩写TC
        taOrgOrder.setTradeNo(CommonUtils.generateTradeNo("TC"));
        taOrgOrder.setAmount(sysVipPackage.getPrice());
        taOrgOrder.setRealAmount(sysVipPackage.getPrice());
        taOrgOrder.setCreateDate(LocalDateTime.now());
        taOrgOrder.setExpiryDate(LocalDateTime.now().plusMinutes(30));
        taOrgOrder.setRemark("线上购买套餐");
        taOrgOrder.setStatus(CommConstant.STATUS_NORMAL);
        taOrgOrder.setTradingStatus(CommConstant.TRADING_STATUS_PROCESSING);
        taOrgOrder.setIsOffline(false);
        taOrgOrder.setUserId(taUser.getUserId());
        taOrgOrder.setUserName(taUser.getUserName());

        return taOrgOrder;
    }

    public TaOrgOrderDetail assembleTaOrgOrderDetail(TaOrgOrder taOrgOrder){
        // 创建订单详情表信息
        TaOrgOrderDetail taOrgOrderDetail = new TaOrgOrderDetail();
        taOrgOrderDetail.setOrderId(taOrgOrder.getOrderId());
        taOrgOrderDetail.setOrgId(taOrgOrder.getOrgId());
        taOrgOrderDetail.setItemType(CommConstant.ITEM_TYPE_PACKAGE);
        taOrgOrderDetail.setAmount(taOrgOrder.getAmount());
        taOrgOrderDetail.setUnitPrice(taOrgOrder.getAmount());
        taOrgOrderDetail.setRealAmount(taOrgOrder.getAmount());
        taOrgOrderDetail.setQuantity(1);
        taOrgOrderDetail.setStatus(CommConstant.STATUS_NORMAL);

        return taOrgOrderDetail;
    }

    public TaUserPackages assembleTaUserPackages(Integer orgId, Integer userId, SaveCzOrderRequestBO saveCzOrderRequestBO, String orderId){
        TaUserPackages taUserPackages = selectPackagesByUserId(userId);
        SysVipPackage sysVipPackage = sysVipPackageMapper.selectById(saveCzOrderRequestBO.getPackId());

        TaUserPackages assembleTaUserPackages = new TaUserPackages();
        assembleTaUserPackages.setOrgId(orgId);
        assembleTaUserPackages.setUserId(userId);
        assembleTaUserPackages.setPackId(saveCzOrderRequestBO.getPackId());
        assembleTaUserPackages.setCreateDate(LocalDateTime.now());
        assembleTaUserPackages.setStartDate(taUserPackages == null ? LocalDateTime.now() : taUserPackages.getEndDate());
        assembleTaUserPackages.setEndDate(calculateEndDate(saveCzOrderRequestBO));
        assembleTaUserPackages.setOrderId(orderId);
        assembleTaUserPackages.setRoleId(sysVipPackage.getRoleId());

        return assembleTaUserPackages;
    }

    private TaUserPackages selectPackagesByUserId(Integer userId){
        QueryWrapper<TaUserPackages> taUserPackagesQueryWrapper = new QueryWrapper<>();
        taUserPackagesQueryWrapper.eq("user_id", userId);
        taUserPackagesQueryWrapper.orderByDesc("end_date");
        List<TaUserPackages> list = taUserPackagesMapper.selectList(taUserPackagesQueryWrapper);
        if (CollectionUtils.isEmpty(list)){
            return null;
        }
        return list.get(0);
    }

    /**
     * 根据选择的套餐计算结束时间
     * @param saveCzOrderRequestBO
     * @return
     */
    private LocalDateTime calculateEndDate(SaveCzOrderRequestBO saveCzOrderRequestBO) {
        LocalDateTime localDateTime = LocalDateTime.now();
        SysVipPackage sysVipPackage = sysVipPackageMapper.selectById(saveCzOrderRequestBO.getPackId());

        switch (sysVipPackage.getUnit()){
            case CommConstant.YEAR:
                localDateTime.plusYears(Long.valueOf(sysVipPackage.getAmount()));
                break;
            case CommConstant.MONTH:
                localDateTime.plusMonths(Long.valueOf(sysVipPackage.getAmount()));
                break;
            case CommConstant.WEEK:
                localDateTime.plusWeeks(Long.valueOf(sysVipPackage.getAmount()));
                break;
            default:
                localDateTime.plusDays(Long.valueOf(sysVipPackage.getAmount()));
        }

        return localDateTime;
    }

}
